@@ -147,6 +147,7 @@ push @virtual_list, qw/
  libtiff-dev
  libuv-dev
  ruby-interpreter
+ perl-xs-dev
  ssh-client
  ssh-server
 /;
@@ -309,6 +310,14 @@ if (not $current_compat_value) {
 
 my $parser ;
 
+after 'store' => sub {
+    my $self = shift;
+    my $data = $self->_fetch_no_check;
+    if ( $data =~ /^debhelper-compat\s*\(\s*=\s*(\d+)\s*\)/ ) {
+        $self->parent->fetch_element(accept_hidden => 1, name => "debhelper-version")->store($1);
+    }
+};
+
 sub dep_parser {
     $parser ||= Parse::RecDescent->new($grammar) ;
     return $parser ;
@@ -335,6 +344,8 @@ sub check_dependency {
     my ($value, $check, $silent, $notify_change, $ok, $apply_fix)
         = @args{qw/value check silent notify_change ok fix/} ;
 
+    return ($ok, $value) if defined $check and $check eq 'no';
+
     # value is one dependency, something like "perl ( >= 1.508 )"
     # or exim | mail-transport-agent or gnumach-dev [hurd-i386]
 
@@ -361,7 +372,7 @@ sub check_dependency {
     my @msgs;
 
     foreach my $dep (@dep_chain) {
-        next unless ref($dep) ; # no need to check variables
+        next unless ref($dep) ; # no need to check debhelper variables
         $self->check_or_fix_pkg_name($apply_fix, $dep, $old, \@msgs) ;
 		$self->check_or_fix_essential_package($apply_fix, $dep, \@msgs) ;
 		$self->check_or_fix_dep($apply_fix, $dep, $old, \@msgs) ;
@@ -754,58 +765,51 @@ sub check_or_fix_dep {
         return;
     }
 
-    if ( $pkg eq 'debhelper' ) {
-        return $self->check_debhelper_version( $apply_fix, $dep_info, $msgs );
-    }
+    # mess with debhelper compat only in Build dependencies
+    if ( $self->element_name ne 'Depends' ) {
+        if ($pkg eq 'debhelper' ) {
+            return $self->check_debhelper_version( $apply_fix, $dep_info, $msgs );
+        }
 
-    if ( $pkg eq 'debhelper-compat' ) {
-        return $self->check_debhelper_compat_version( $apply_fix, $dep_info, $msgs );
+        if ( $pkg eq 'debhelper-compat' ) {
+            return $self->check_debhelper_compat_version( $apply_fix, $dep_info, $msgs );
+        }
     }
 
     my ( $vers_dep_ok, @list ) =  $self->check_versioned_dep( $dep_info );
     return if $vers_dep_ok;
 
+    my ($warn_str, $log_str, $fix_sub);
     if ($dep_info->{dep}[0] =~ /</) {
-        $self->warn_or_remove_dep ($apply_fix, $dep_info, \@list, $msgs) ;
+        $warn_str = "unnecessary older-than versioned dependency: ". $self-> struct_to_dep($dep_info)
+            . ". Debian has @list";
+        $log_str = "removed dependency $dep_info->{name}";
+        # the fix removes the whole dependency
+        $fix_sub =  sub{ $_[0]->%* = () };
     }
     else {
-        $self->warn_or_remove_vers_dep ($apply_fix, $dep_info, \@list, $msgs) ;
+        $warn_str = "unnecessary greater-than versioned dependency: ". $self-> struct_to_dep($dep_info)
+            . ". Debian has @list";
+        $log_str = "removed greater-than versioned dependency from $dep_info->{name}";
+        # the fix removes greater-than versioned dep, notify_change is called in check_value
+        $fix_sub = sub { delete $_[0]->{dep} };
     }
+    $self->warn_or_fix_dep_info ($fix_sub, $apply_fix, $dep_info, $msgs, $warn_str, $log_str) ;
 }
 
-sub warn_or_remove_vers_dep {
-    my ( $self, $apply_fix, $dep_info, $list, $msgs ) = @_;
-    my $pkg = $dep_info->{name};
 
-    my $msg = "unnecessary greater-than versioned dependency: ". $self-> struct_to_dep($dep_info)
-        . ". Debian has @$list";
+sub warn_or_fix_dep_info {
+    my ( $self, $fix_sub, $apply_fix, $dep_info, $msgs, $warn_msg, $log_str ) = @_;
+
     if ($apply_fix) {
-        delete $dep_info->{dep};    # remove greater-than versioned dep, notify_change called in check_value
-        $logger->info("fix: removed greater-than versioned dependency from $pkg");
-        push $msgs->@*, $msg;
+        $fix_sub->($dep_info);
+        $logger->info("fix: $log_str");
+        push $msgs->@*, $warn_msg;
     }
     else {
         $self->{nb_of_fixes}++;
-        $self->add_warning( $msg);
-        $logger->info("will warn: $msg (fix++)");
-    }
-}
-
-sub warn_or_remove_dep {
-    my ( $self, $apply_fix, $dep_info, $list, $msgs ) = @_;
-    my $pkg = $dep_info->{name};
-
-    my $msg = "unnecessary older-than versioned dependency: ". $self-> struct_to_dep($dep_info)
-        . ". Debian has @$list";
-    if ($apply_fix) {
-        %$dep_info = ();    # remove whole dependency
-        $logger->info("fix: removed dependency $pkg");
-        push $msgs->@*, $msg;
-    }
-    else {
-        $self->{nb_of_fixes}++;
-        $self->add_warning( $msg);
-        $logger->info("will warn: $msg (fix++)");
+        $self->add_warning( $warn_msg );
+        $logger->info("will warn: $warn_msg (fix++)");
     }
 }
 
