@@ -4,19 +4,21 @@ use strict;
 use warnings;
 
 use 5.20.0;
-use IO::Pipe;
 use Exporter::Lite;
 use Array::IntSpan;
 use Path::Tiny;
+use Time::localtime;
 use Carp;
-use YAML::Tiny;
+use YAML::XS qw/LoadFile/;
+
+$YAML::XS::LoadBlessed = 0;
 
 use feature qw/postderef signatures/;
 no warnings qw/experimental::postderef experimental::signatures/;
 
-binmode STDOUT, ':utf8';
+binmode STDOUT, ':encoding(utf8)';
 
-our @EXPORT = qw(scan_files print_copyright);
+our @EXPORT = qw(scan_files print_copyright __from_copyright_structure __to_copyright_structure);
 
 my $whitespace_list_delimiter = $ENV{'whitespace_list_delimiter'} || "\n ";
 
@@ -25,7 +27,7 @@ my $whitespace_list_delimiter = $ENV{'whitespace_list_delimiter'} || "\n ";
 # derivative work to LGPL-2.1+ was given by Jonas.
 # see https://lists.alioth.debian.org/pipermail/pkg-perl-maintainers/2015-March/084900.html
 
-# Copyright 2014 Dominique Dumont <dod@debian.org>
+# Copyright 2014-2020 Dominique Dumont <dod@debian.org>
 # Copyright © 2005-2012 Jonas Smedegaard <dr@jones.dk>
 # Description: Reformat licencecheck output to copyright file format
 #
@@ -52,7 +54,7 @@ sub print_copyright ( %args ) {
     }
 
     # regroup %files hash: all leaves have same id -> wild card
-    __squash($split_files);
+    __squash($split_files, $copyrights_by_id);
 
     # pack files by copyright id
     my @packed = __pack_files($split_files);
@@ -61,7 +63,7 @@ sub print_copyright ( %args ) {
 
     foreach my $p (@packed) {
         my ($id, @paths) = $p->@*;
-        my ($c,$l) = $copyrights_by_id->[$id]->@*;
+        my ($c,$l) = __from_copyright_structure($copyrights_by_id->[$id]);
 
         next if $id == 0;
 
@@ -96,18 +98,20 @@ $default{ignore}= << 'EOR';
 # Ignore general backup files
 ~$|
 # Ignore emacs recovery files
-(?:^|/)\.#|
+(^|/)\.#|
 # Ignore vi swap files
-(?:^|/)\..*\.swp$|
+\.swp$|
 # Ignore baz-style junk files or directories
-(?:^|/),,.*(?:$|/.*$)|
+(^|/),,.*(?:$|/.*$)|
 # File-names that should be ignored (never directories)
-(?:^|/)(?:DEADJOE|\.cvsignore|\.arch-inventory|\.bzrignore|\.gitignore)$|
+(^|/)(DEADJOE|\.cvsignore|\.arch-inventory|\.bzrignore|\.gitignore)$|
 # File or directory names that should be ignored
-(?:^|/)(?:CVS|RCS|\.pc|\.deps|\{arch\}|\.arch-ids|\.svn|\.hg|_darcs|\.git|
+(^|/)(CVS|RCS|\.pc|\.deps|\{arch\}|\.arch-ids|\.svn|\.hg|_darcs|\.git|
 \.shelf|_MTN|\.bzr(?:\.backup|tags)?)(?:$|/.*$)|
 # skip debian files that are too confusing or too short
-(?:^|/)debian/((fix.scanned.)?copyright|changelog|NEWS|compat|.*yml|docs|source|patches/series)
+(?:^|/)debian/((fix.scanned.)?copyright|changelog|NEWS|compat|.*yml|docs|source|patches/series)|
+# skip some binary files
+(png|jpg|pdf|ico|bmp|jpe?g)$
 EOR
 
 # also from licensecheck
@@ -158,18 +162,14 @@ map { s/#.*\n//g;  s/[\s\n]+//g; } values %default;
 # option to skip UNKNOWN ?
 # load a file to override some entries ?
 sub scan_files ( %args ) {
-
     $quiet = $args{quiet} // 0;
 
     my $current_dir = $args{from_dir} || path('.');
 
-    my %regexps = %default; # default is needed during tests 
+    my %regexps = %default; # default is needed during tests
     my @lines ;
-    if (my $file = $ENV{COPYRIGHT_SCANNER_INPUT}) {
-        @lines = path($file)->lines_utf8 ; # for tests
-    }
-    elsif ($args{in}) {
-        @lines = $args{in}->lines_utf8; # for other tests
+    if ($args{in}) {
+        @lines = $args{in}->lines_utf8; # for tests
     }
     else {
         my $scan_data = {};
@@ -177,31 +177,31 @@ sub scan_files ( %args ) {
         my $scan_patterns = $debian->child("copyright-scan-patterns.yml");
 
         if ($debian->is_dir and $scan_patterns->is_file) {
-            my $yaml = $scan_patterns->slurp_utf8;
-            # { check => { suffixes => [ js pl ], pattern => '[A-Z]*'}, ignore => { suffixes => [ jpg png ], pattern => 'foo.*'} }
-            $scan_data = Load $yaml;
+            $scan_data = LoadFile($scan_patterns->stringify);
         }
 
         # licensecheck --check is broken ( #842368 ), so --skipped option is useless.
         # let's scan everything and skip later
         foreach my $what (qw/check ignore/) {
-            my $data = $scan_data->{$what} or next;
+            my $data = $scan_data->{$what} || {};
             my $reg = join(
                 '|' ,
                 (map { '\.'.$_.'$'} @{$data->{suffixes} || []}),
                 @{ $data->{pattern} || []},
                 $default{$what}
             );
-            $regexps{$what} = qr/$reg/;
+            $regexps{$what} = $reg;
         }
 
-        my $pipe = IO::Pipe->new();
-        my @cmd = (qw/licensecheck --encoding utf8 --copyright --machine --deb-fmt --recursive/,
-                   '--ignore='.$regexps{ignore}, '.');
-        $pipe->reader(@cmd);
+        my $lic_cmd = 'licensecheck --encoding utf8 --copyright --machine --deb-fmt --recursive '
+            . "'--ignore=%s' %s";
+        my $cmd = sprintf($lic_cmd, $regexps{ignore}, $current_dir);
+        open(my $pipe, "-|", $cmd) or die "Can't open '$cmd' : $!\n";;
         binmode($pipe, ":encoding(UTF-8)");
-        @lines = $pipe->getlines;
-        $pipe->close;
+        my $length = length($current_dir->stringify) + 1;
+        @lines = map { substr $_,$length } $pipe->getlines;
+        $pipe->close or die $! ? "Error closing licensecheck pipe: $!\n"
+                               : "Exit status ".$?." $cmd\n";
     }
 
     my $fill_blank_data = __load_fill_blank_data($current_dir);
@@ -224,6 +224,18 @@ sub scan_files ( %args ) {
         # and it's a collective work by nature.
         next if $f =~  m!debian/(copyright|changelog)!;
 
+        # skip copyright infos extracted from license files when they
+        # come from the author of the license and not from the project
+        # author
+        if ($f =~  m/^COPYING|LICENSE$/) {
+            if ($c =~ /Free Software Foundation/) { next; }
+            if ($c =~ /Perl Foundation/) { next; }
+        }
+
+
+        # skip license files without copyright information (usually plain license)
+        next if $f =~  m/^COPYING|LICENSE$/ && $c =~ /No copyright/;
+
         # filter out files user does not want checked
         if ( $f !~ $regexps{check} ) {
             # the ignore test should not be usefull as the files are ignored by licensecheck
@@ -231,32 +243,6 @@ sub scan_files ( %args ) {
             next;
         }
 
-        # $l =~ s/([*?\\])/\\$1/g;
-        # $l =~ s/\s*\(unversioned\/unknown version\)//g;
-        # $l =~ s/\s*\(with incorrect FSF address\)//g;
-        # $l =~ s/(\w+)\s+\(v([^)]+) or v([^)]+)\)/uc($1)."-$2 or ".uc($1)."-$3"/eg;
-        # $l =~ s/\s+\(v([^)]+) or later\)/-$1+/g;
-        # $l =~ s/\s+\(v([^)]+)\)/-$1/g;
-        # $l =~ s/^\s*(GENERATED FILE)/UNKNOWN/g;
-        # $l =~ s/\s+(GENERATED FILE)//g;
-        # $l =~ s/\bzlib\/libpng\b/Zlib/g;
-        # $l =~ s/\bMIT\/X11 \(BSD like\)/Expat/g;
-        # $l =~ s/\bBSD \((\d) clause\)/BSD-$1-clause/g;
-        # $l =~ s/\bpublic domain\b/public-domain/ig;
-
-        # licensecheck uses and/or when a file contains 2
-        # licenses. and/or is not a valid syntax for debian/copyright
-        # file
-        $l =~ s!and/or!or!g;
-
-        # # deduplicate license keywords and add 'or' keyword where missing
-        # # this is fragile. may need to change license-check to output license keyword
-        # my %tmp_l = map { ( $_ => 1) ;} grep { $_ ne 'or' } split(/ /,$l);
-        # $l = join(' or ', sort keys %tmp_l) ;
-
-        # Handle license like "under the same term as Perl"
-        # $l =~ s/perl/Artistic or GPL-1+/i;
-
         $c =~ s/'//g;
         $c =~ s/^&copy;\s*//g;
         $c =~ s/\(c\)\s*//g;
@@ -270,6 +256,9 @@ sub scan_files ( %args ) {
         $c =~ s!^[\s,/*]|[\s,#/*-]+$!!g;
         $c =~ s/--/-/g;
         $c =~ s!\s+\*/\s+! !;
+        # libuv1 has copyright like "2000, -present"
+        $c =~ s![,\s]*-present!'-'.(localtime->year() + 1900)!e;
+        $c =~ s![\x00-\x1f].*!!; # cut off everything after and including the first non-printable
 
         $c = __pack_copyright($c);
 
@@ -338,7 +327,7 @@ sub scan_files ( %args ) {
     foreach my $c (sort keys %copyrights) {
         foreach my $l (sort keys $copyrights{$c}->%* ) {
             my $id = $copyrights{$c}{$l};
-            $copyrights_by_id[$id] = [ $c, $l ] ;
+            $copyrights_by_id[$id] = __to_copyright_structure( $c, $l ) ;
         }
     }
 
@@ -353,6 +342,17 @@ sub scan_files ( %args ) {
     return ($files, \@copyrights_by_id) ;
 }
 
+sub __to_copyright_structure ($c, $l) {
+    return {
+        Copyright => $c,
+        License => { short_name => $l},
+    };
+}
+
+sub __from_copyright_structure ($s) {
+    return ($s->{Copyright}, $s->{License}{short_name});
+}
+
 sub __split_copyright ($c) {
     my ($years,$owner) = $c =~ /^(\d\d[\s,\d-]+)(.*)/;
     # say "undef year in $c" unless defined $years;
@@ -448,7 +448,7 @@ sub __squash_copyrights_years ($copyrights_by_id) {
 
     my %id_year_by_same_owner_license;
     for (my $id = 1; $id < $copyrights_by_id->@* ; $id++ ) {
-        my ($c,$l) = $copyrights_by_id->[$id]->@* ;
+        my ($c, $l) = __from_copyright_structure($copyrights_by_id->[$id]);
         #say "id $id: c $c l $l";
         my @owners ;
         my @years ;
@@ -480,7 +480,7 @@ sub __squash_copyrights_years ($copyrights_by_id) {
         # store (c) info with coalesced years in new item of $copyrights_by_id
         my $new_id = $copyrights_by_id->@* ;
         my $new_cop = join("\n ",@squashed_c) ;
-        $copyrights_by_id->[$new_id] = [ $new_cop , $l ];
+        $copyrights_by_id->[$new_id] = __to_copyright_structure( $new_cop , $l );
         #say "created id $new_id with c $new_cop l $l";
         # fill the swap table entry-id -> coaslesces entry-id
         foreach my $id ( map { $_->[0]} @entries) {
@@ -535,26 +535,47 @@ sub __coalesce_copyright_years($entries, $owners) {
 
 # $h is a tree of hash matching the directory structure. Each leaf is a
 # copyright id. Each key is a file name in a directory (not the full path)
-sub __squash ($h, $p = '') {
+sub __squash ($h, $copyrights_by_id, $p = '') {
     my %count ;
 
     # find main license info found in LICENSE or COPYING or README
-    # file. Order is important. Info found in README trumps info found
-    # in LICENSE, which trumps info found in COPYING. For instance,
-    # when info is found in README, the files COPYING and LICENSE are
-    # handled as regular files.
-    my $found_main_info = 0;
+    #  file. LICENSE and COPYING are removed from the file list as
+    #  they specify the license author as copyright owner
+    my %main_infos;
     foreach my $info_name (qw/README LICENSE LICENCE COPYING/) {
         my $re = qr!^$info_name[.\w]*$!;
         foreach my $name (sort keys %$h) {
             if ($name =~ $re) {
                 my $id = $h->{$name};
+                delete $h->{$name} if $info_name ne 'README';
                 next unless $id; # skip files without info
-                $count{$id} = 1;
-                $found_main_info++;
+                $main_infos{$id} = 1;
             };
         }
-        last if $found_main_info;
+    }
+
+    my $main_license_id;
+    my @main_info_keys = sort keys %main_infos;
+    if (@main_info_keys == 1) {
+        $main_license_id = $main_info_keys[0];
+    }
+    elsif (@main_info_keys > 1) {
+        # create a new copyright id containing a sum of all info found
+        # in README LICENSE (...) files (if they are different and
+        # have actual info)
+        my %licenses;
+        my %copyrights;
+        foreach my $info_id (@main_info_keys) {
+            my ($cop, $lic) = __from_copyright_structure($copyrights_by_id->[$info_id]);
+            $licenses{$lic} = 1 unless $lic =~ /unknown/i;
+            $copyrights{$cop} = 1 unless $cop =~ /unknown|no-info-found/i;;
+        }
+        push $copyrights_by_id->@*,
+            __to_copyright_structure (
+                join("\n", sort keys %copyrights),
+                join(" or ", sort keys %licenses)
+            );
+        $main_license_id = $#$copyrights_by_id;
     }
 
     # count the number of times each (c) info is used in this directory.
@@ -564,7 +585,7 @@ sub __squash ($h, $p = '') {
         if (ref($item)) {
             # squash may return a plain id, or a hash with '*' => id ,
             # or a non squashable hash
-            $h->{$name} = __squash($item, $p.'/'.$name);
+            $h->{$name} = __squash($item, $copyrights_by_id, $p.'/'.$name);
         }
         my $id = (ref($item) and defined $item->{'*'}) ? $item->{'*'} : $item ;
 
@@ -576,10 +597,9 @@ sub __squash ($h, $p = '') {
         }
     }
 
-    # find the most used (c) info in this directory (or the existing '*' entry)
+    # find the most used (c) info in this directory
     # unless info was already found in LICENSE or README content
     my $max = 0;
-    my $main_license_id = $h->{'*'};
     if (not defined $main_license_id) {
         foreach my $id (sort keys %count) {
             if ($count{$id} > $max) {
@@ -621,8 +641,7 @@ sub __load_fill_blank_data ($current_dir) {
 
         warn "Note: loading @fills fixes" if @fills and not $quiet;
         foreach my $file ( @fills) {
-            my $yaml = $file->slurp_utf8;
-            my $data = Load $yaml;
+            my $data = LoadFile($file->stringify);
             foreach my $path (sort keys %$data) {
                 if ($fill_blanks{$path}) {
                     warn "Warning: skipping duplicated fill blank path $path from file $file";
@@ -681,18 +700,43 @@ license information. Information are packed in a way to ease review and
 maintenance. Files information is grouped with wildcards ('*') to reduce
 the list of files.
 
+=head2 About LICENSE and README files
+
+Projects often store global copyright information, i.e. information
+that apply to all files of a project (unless specified otherwise in
+some files) in README or LICENSE or COPYING file.
+
+The information contained in these files id merged and applied to the
+directory entry that contain them.
+
+I.e files like:
+
+ foo_comp/README: (c) 2018 Joe, GPL-2
+ foo_comp/LICENSE: (c) 2017 Max, GPL-3
+
+yield the following copyright entries:
+
+ foo_comp/*:
+ Copyright: 2018 Joe
+   2019 Max
+ License: GPL-2 or GPL-3
+
+ README:
+ Copyright: 2018 Joe
+ License: GPL-2
+
 =head1 Selecting or ignoring files to scan
 
 By default, scanner scans source files with known suffixes (like .c
 .pl ...), README, scripts and skip backup files.
 
-You can change this behavior in file
-C<debian/copyright-scan-patterns.yml>. This YAML file can contain a
+If needed, this behavior can tuned with
+C<debian/copyright-scan-patterns.yml> file. This YAML file contains a
 list of suffixes or patterns to scan or to ignore that are added to
 the default list. Any file that is not scanned or ignored will be
 shown as "skipped".
 
-The file must have the following structure (all fields are optional
+This file must have the following structure (all fields are optional
 and order does not matter):
 
  ---
@@ -836,8 +880,8 @@ set to 1 to suppress progress messages. Should be used only in tests.
 
 The output of L<licensecheck> is expected to be utf-8. Which means
 that the source files scanned by L<licensecheck> should also be
-encoded in utf-8. In practice, this will impact only copyright owner
-name which may be garbled if comments are not encoded in utf-8.
+encoded in utf-8. This program will abort if invalid utf-8 characters
+are found.
 
 =head1 BUGS
 
@@ -845,14 +889,14 @@ Extracting license and copyright data from unstructured comments is not reliable
 User must check manually the files when no copyright info is found or when the
 license is unknown.
 
-Source files are assumed to be utf8 (or ascii). Using files with other encoding
-will lead to garbled names of copyright owner. In this case, you can:
+Source files are assumed to be utf8 (or ascii). Using files with invalid characters
+will break C<cme>. In this case, you can:
 
 =over
 
 =item *
 
-Path source files to use utf-8 encoding.
+Patch source files to use utf-8 encoding.
 
 =item *
 
@@ -862,7 +906,7 @@ encoding.
 
 =item *
 
-File a bug against licensecheck of devscript package to find a better solution.
+File a bug against licensecheck package to find a better solution.
 
 =back
 
